# =============================================================================
# =============================================================================
# DAMAGE ASSESSMENT USING PELICUN
# By: Gerald Palomino (08/08/2022)
# =============================================================================
# =============================================================================

# Implementation from the models in the following paper:
# [1] G.Palomino, K.Blowes, and C. Molina Hutt, "Evaluating post-earthquake 
# functionality and surge capacity of hospital emergency departments using
# discrete event simulation," Earthquake Spectra, vol. In review, 2022.

# Pelicun model (e.g., DL_model-2directions.json) must be located on current directory


# =============================================================================
# Define path & Import modules
# =============================================================================

path = r'C:\Users\geral\Documents\Research\_doc\_Electronic Supplement\S2_Pelicun_Model_(building_performance_model)'

# inputs
EDPs_file = path+'\EDP_VGH_M9\EDP_VGH_M9.csv'
DL_pelicun_model = "DL_model-2directions.json"

# outputs
resuts_temp = path+'\outputs_temporal'
results_path = path+'\outputs_VGH_M9'

# modules
import os
import DL_calculation
import csv
import shutil

aa=os.getcwd()
shutil.rmtree(results_path)
os.mkdir(results_path)

# =============================================================================
# Get Header
# =============================================================================
numStory=4
header = ['']
for i in range(numStory):
    header.append('1-PID-'+str(i+1)+'-1')
    header.append('1-PID-'+str(i+1)+'-2')
for i in range(numStory+1):
    header.append('1-PFA-'+str(i)+'-1')
    header.append('1-PFA-'+str(i)+'-2')  
header.append('1-RID-1-1')
header.append('1-RID-1-2')

# =============================================================================
# Read M9 EDPs
# =============================================================================

original_EDP_file = EDPs_file
with open(original_EDP_file, 'r') as rf:
    next(rf)
    reader = csv.reader(rf, delimiter=',')
    data = list(reader)


# =============================================================================
# Run Simlutaions
# =============================================================================

GM_first = 1
GM_last = 30
NumSimulations = (GM_last - GM_first) + 1


for GM_number in range(GM_first, GM_last+1):
    
    # Update M9 file
    with open("EDP_VGH_M9_temp.csv", "w", newline='') as fp:
        wr = csv.writer(fp, dialect='excel')
        wr.writerow(header)
        wr.writerow(data[GM_number-1])

    loss=DL_calculation.FEMA_P58_Assessment()
    loss.read_inputs(DL_pelicun_model,"EDP_VGH_M9_temp.csv",verbose=False)
    loss.define_random_variables()
    loss.define_loss_model()
    loss.calculate_damage()
    loss.calculate_losses()
    loss.aggregate_results()
    loss.save_outputs(resuts_temp, 'EDP.csv', 'DM.json', 'DV.json')
    
    # Save outputs
    src_dir = resuts_temp
    dst_dir = results_path+'\GM_'+str(GM_number)
    shutil.copytree(src_dir,dst_dir)
    
    # Print msg
    print ('GM_'+str(GM_number)+' done')

