# =============================================================================
# =============================================================================
# FUNCTIONALITY MEDIAN SCENARIO FINDER
# By: Gerald Palomino (08/08/2022)
# =============================================================================
# =============================================================================

# Implementation from the models in the following paper:
# [1] G.Palomino, K.Blowes, and C. Molina Hutt, "Evaluating post-earthquake 
# functionality and surge capacity of hospital emergency departments using
# discrete event simulation," Earthquake Spectra, vol. In review, 2022.

# Functionality factors file per Ground Motion (e.g., Func_GM_1.csv) must be 
# located on current directory

# To plot results, copy results from FunctionalityFactors.txt to Functionality_Results.csv

# =============================================================================
# Define path & Import modules
# =============================================================================

path = r'C:\Users\geral\Documents\Research\_doc\_Electronic Supplement\S3_Functionality_Loss_Calculations\Functionality_results'

# inputs
functionality_combos = path +'\Functionality_Combinations.csv'

# outputs
mediam_functionality_factors = "FunctionalityFactors.txt"
functionality_plot = path+'\Functionality_plot'


# modules
import numpy as np
import pandas as pd
import csv
import shutil
import matplotlib.pyplot as plt

# =============================================================================
# Functions
# =============================================================================

def get_subtotal(array, Subtotal, MC_simulations):
    count = 0
    for i in range(MC_simulations):
        comparison = data[i] == array
        equal_arrays = comparison.all()
        if equal_arrays == 1:
            count = count+1
    Subtotal = Subtotal + count
    print(str(array.tolist())+' - '+str(count)+' times')
    return Subtotal

# =============================================================================
# Find median Scenario to estimate Functionality
# =============================================================================

file = functionality_combos
# Read file
a = pd.read_csv(file,sep=",",skiprows=0)
a = a.drop(a.columns[0], axis=1)
a = a.drop(a.columns[4], axis=1)
Combinations = a.values

GM_first = 1
GM_last = 30
NumSimulations = (GM_last - GM_first) + 1
MC_simulations = 1000
output =[]

for GM_number in range(GM_first, GM_last+1):
    
    # Read file
    file = path+'\Func_GM_'+str(GM_number)+'.csv'
    a = pd.read_csv(file,sep=",",skiprows=0)
    a = a.drop(a.columns[0], axis=1)
    data = a.values
    
    print ('-----------------------------------------')
    print ('GM_'+str(GM_number))
    
    Subtotal = 0
    i=0

    while Subtotal < MC_simulations/2:
        array = Combinations[i]
        Subtotal = get_subtotal(array, Subtotal, MC_simulations)
        i=i+1
    print('Median value: '+str(array.tolist())+' - Subtotal: '+str(Subtotal))
    scenario = np.array(GM_number)
    array = np.append(scenario,array)
    output.append(array)
    
# =============================================================================
# Write Output File  
# =============================================================================
    
output_file = mediam_functionality_factors
np.savetxt(output_file, output, header = 'Scenario\t1st floor\t2nd floor\t3rd floor\t4th floor', fmt='%s', delimiter='\t')


# =============================================================================
# Plot Functionality for 30 Simulations
# =============================================================================

file2 = path +'\Functionality_Results.csv'
# Read file
a = pd.read_csv(file2,sep=",",skiprows=0)
a = a.drop(a.columns[0], axis=1)
a = a.drop(a.columns[4], axis=1)
Funct = a.values*100

GM_first = 1
GM_last = 30
NumSimulations = (GM_last - GM_first) + 1

# Plots
Ygrid = [1,2,3,4]
plt.rcParams["figure.figsize"] = (3.25,2.20)
plt.rcParams['axes.linewidth'] = 1.25 
titles_font = {'fontname':'Arial'}
media = [0,0,0,0]

for GM_number in range(GM_first, GM_last+1):
    plt.plot(Funct[GM_number-1],Ygrid, color = '0.6', alpha =0.3, linewidth=1.0)
    media = media + Funct[GM_number-1]
 
media = media / NumSimulations
plt.plot(media, Ygrid, color = 'black', linewidth=2.0, label = 'Avg. Functionality')
    
plt.xlabel('Functionality, %', fontsize=10, **titles_font)
plt.ylabel('Stories', fontsize=10, **titles_font)
# plt.title('Wait Times - Scenario '+str(Scenario_number), fontsize=10, **titles_font)
leg = plt.legend(fontsize=8, loc="lower left")
leg.get_frame().set_edgecolor('0.0')
leg.get_frame().set_linewidth(0.75)
# plt.grid()
plt.xticks([0,25,50,75,100], fontsize=8)
plt.yticks([1,2,3,4], fontsize=8)
plt.xlim([0,120])
plt.ylim([1,4])
path = functionality_plot
plt.savefig(path, bbox_inches="tight", dpi =300)
plt.show()
