# =============================================================================
# =============================================================================
# FUNCTIONALITY ASSESSMENT - RUN EXCEL MACRO FOR VGH HOSPITAL
# By: Gerald Palomino (08/08/2022)
# =============================================================================
# =============================================================================

# Implementation from the models in the following paper:
# [1] G.Palomino, K.Blowes, and C. Molina Hutt, "Evaluating post-earthquake 
# functionality and surge capacity of hospital emergency departments using
# discrete event simulation," Earthquake Spectra, vol. In review, 2022.

# "JPP_functionality.xlsm" file must be located in the current directory 


# =============================================================================
# Define path & Import modules
# =============================================================================

#path
path = r'C:\Users\geral\Documents\Research\_doc\_Electronic Supplement'

# inputs
damage_results_path = path+'\S2_Pelicun_Model_(building_performance_model)\outputs_VGH_M9'

#outputs
functionality_results_path = path+'\S3_Functionality_Loss_Calculations\Functionality_results'

# modules
import os, os.path
import win32com.client
from shutil import copyfile
import pandas as pd

GM_first = 1
GM_last = 30
NumSimulations = (GM_last - GM_first) + 1
MC_simulations = 1000


for GM_number in range(GM_first, GM_last+1):

    # Update Pelicun file (reference file)
    src = damage_results_path+'\GM_'+str(GM_number)+'\DMG.csv'
    # add residual drift column
    RID_data = damage_results_path+'\GM_'+str(GM_number)+'\EDP_.csv'
    RID_cols = pd.read_csv(RID_data, delimiter=',',usecols=[19,20]) # RID in 2 directions is on columns 19, 20
    new_row = pd.DataFrame({'1-RID-1-1':[0,0,0], '1-RID-1-2':[0,0,0]})
    RID_cols = new_row.append(RID_cols, ignore_index=True)
    aa = pd.read_csv(src, delimiter=',')
    xd = aa.merge(RID_cols, left_index = True, right_index = True)
    xd.to_csv(src, index = False)
    dst = damage_results_path+'\DMG.csv'
    copyfile(src, dst)

    # Run Macro (excel file)
    if os.path.exists("JPP_functionality.xlsm"):
        xl=win32com.client.Dispatch("Excel.Application")
        xl.Workbooks.Open(os.path.abspath("JPP_functionality.xlsm"), ReadOnly=1)
        xl.Application.Run("JPP_functionality.xlsm!Module2.UpdateSource")
        xl.Application.Run("JPP_functionality.xlsm!Module3.Functionality_calc")
        # xl.Application.Save() # if you want to save then uncomment this line and change delete the ", ReadOnly=1" part from the open function.
        xl.Application.Quit() # Comment this out if your excel script closes
        del xl

    # Save oututs
    read_file = 'JPP_functionality.xlsm'
    output_file = functionality_results_path+'\Func_GM_'+str(GM_number)+'.csv'
    # Read file
    a = pd.read_excel(read_file,sheet_name='Results',skiprows=2)
    a = a.drop(a.columns[6], axis=1)
    a = a.drop(a.columns[5], axis=1)
    a.to_csv(output_file, sep=',', index=False)
    # data = a.values
    print ('GM_'+str(GM_number)+' done')

