# =============================================================================
# =============================================================================
# DISCRETE EVENT SIMULATION MODEL IN JAAMSIM - RESULTS
# By: Gerald Palomino (08/08/2022)
# =============================================================================
# =============================================================================

# Implementation from the models in the following paper:
# [1] G.Palomino, K.Blowes, and C. Molina Hutt, "Evaluating post-earthquake 
# functionality and surge capacity of hospital emergency departments using
# discrete event simulation," Earthquake Spectra, vol. In review, 2022.

# In the first line of 'VGH_Emergency_Department_capacity_python.cfg' file,
# update path for 'VGH_Emergency_Department_capacity1.cfg' file 

# Update directory of 'JaamSim2020-12.jar' in Line 94 


# =============================================================================
# Define path & Import modules
# =============================================================================

# inputs
functionality_file = r'C:\Users\geral\Documents\Research\_doc\_Electronic Supplement\S2_Pelicun_Model_(building_performance_model)\Functionality_results\Functionality_results.csv'

# outputs
results_plots_path = r'C:\Users\geral\Documents\Research\_doc\_Electronic Supplement\S4_Discrete_Event_Simulation_Model_in_JaamSim\Results_plots'

# modules
import subprocess
import os
import os.path
import numpy as np
import matplotlib.pyplot as plt
import csv
import re
import pandas as pd
import math
import statistics

# =============================================================================
# Functions
# =============================================================================

def format_parameters1(values_to_set):
    changed_values=''
    for item in values_to_set:
        changed_values+='%s {%s}\n'%(item[0],item[1])
    changed_values=changed_values[:-1]
   
    return changed_values

def format_parameters2(values_to_set):
    changed_values=''
    for item in values_to_set:
        changed_values+='%s {%s min}\n'%(item[0],item[1])
    changed_values=changed_values[:-1]

def format_parameters3(values_to_set):
    changed_values=''
    for item in values_to_set:
        changed_values+='%s { {0  d  %s} {22  d  %s} {22.01  d  %s} {35  d  %s} }\n'%(item[0],item[1],item[1],item[2],item[2])
    changed_values=changed_values[:-1]
   
    return changed_values

def get_modified_script(script, parameters1, parameters3):
    with open(script, "r") as f:
        include_base_model = f.readlines()[0]
    with open(script, "w") as f:
        f.write(include_base_model)
        f.write('\n')
        f.write(format_parameters1(parameters1))
        # f.write('\n')
        # f.write(format_parameters2(parameters2))
        f.write('\n')
        f.write(format_parameters3(parameters3))
    return
    

def get_directory(first_sub,second_sub,file_name):
    
    current_directory = os.path.dirname(os.path.realpath('__file__'))
    
    parent_directory = os.path.split(os.path.split(current_directory)[0])[0] + "/"
    
    file_patch = os.path.join(parent_directory,first_sub,second_sub,file_name)
    file_path_custom = file_patch.replace("/","\\")
    
    return file_path_custom

   
def run_simulation(Jaamsim_model):
    # C:\Users\geral\Documents\Research\JaamSim
    # subprocess.call(['java','-jar', get_directory("JaamSim","Capacity_Estimation","JaamSim2020-12.jar"), Jaamsim_model,'-h'])
    subprocess.call(['java','-jar', get_directory("_Electronic Supplement","S4_Discrete_Event_Simulation_Model_in_JaamSim","JaamSim2020-12.jar"), Jaamsim_model,'-h'])


def get_waiting_times(WT_file, N_points, i):
    time=[]
    OR_WT=[]
    ER_WT=[]
    Beds_WT=[]
    Triage_WT=[]
    Imaging_WT=[]
    Diagnostic_WT=[]
    Demand_SS=[]
    Demand_EQ=[]
    Demand_Total=[]
       
    with open(WT_file, 'r') as f:
        a = i*(N_points+2)+2
        b = a+N_points
        # a = 2
        # b = a + N_points
        reader = f.readlines()[a:b]
        for line in reader:
            row = line.split('\t')
            time.append((float(row[0])-20*24)/24) #h to days
            OR_WT.append(float(row[1]))
            ER_WT.append(float(row[2]))
            Beds_WT.append(float(row[3]))
            Triage_WT.append(float(row[4]))
            Imaging_WT.append(float(row[5]))
            Diagnostic_WT.append(float(row[6]))
            Demand_SS.append(float(row[7]))
            Demand_EQ.append(float(row[8]))
            Demand_Total.append(float(row[9]))
            
            
    return time, OR_WT, ER_WT, Beds_WT, Triage_WT, Imaging_WT, Diagnostic_WT, Demand_SS, Demand_EQ, Demand_Total


def get_avg_LOS(LOS_file):
    time=[]
    total_LOS=[]
       
    with open(LOS_file, 'r') as f:
        a = 11
        # b = a+N_points
        reader = f.readlines()[a:]
        for line in reader:
            row = line.split('\t')
            time.append((float(row[0])-20*24)/24) #h to days
            total_LOS.append(float(row[3]))
    
    num_patients = len(time)
    interval = 12 # calculate avg every 24h
    avg_LOS = []
    avg_time = []
    time2 = time
    total_LOS2 = total_LOS
    for i in range (int(30*24/interval-1)):
        avg_time.append(i*interval)
        temp = 0
        Sum = 0
        while time2[temp] < ( (i+1)*interval/24 ) :
            Sum = total_LOS2[temp] + Sum    
            temp = temp+1
        avg_LOS.append(Sum/temp)
        time2 = time2[temp:]
        total_LOS2 = total_LOS2[temp:]
    
    # adding one more point
    avg_time.append(i*interval+interval)
    avg_LOS.append(Sum/temp)
    
    return avg_time, avg_LOS, num_patients


def plot_LOS(time, LOS_S1, LOS_S2, LOS_S3, LOS_total, Scenario_number):
    
    titles_font = {'fontname':'Arial'}
    
    time2 = [t/24 for t in time]
    plt.plot(time2, LOS_S1, color = 'tab:green', linewidth=1.0, label = 'Green Codes')
    plt.plot(time2, LOS_S2, color = 'gold', linewidth=1.0, label = 'Yellow Codes')
    plt.plot(time2, LOS_S3, color = 'tab:red', linewidth=1.0, label = 'Red Codes')
    plt.plot(time2, LOS_total, color = 'black', linewidth=2.0, label = 'Total LOS')
    plt.plot([0,30], [10,10], color = 'black', linewidth=2.0, linestyle='--', label = 'Limit')
    
    # EQ occurs at day 2
    plt.plot([2,2], [0,500], color = 'black', linewidth=0.75, linestyle='--') 
    l1 = np.array((2.7, 17.6)) # 3.4 / 15.8
    plt.text(*l1, 'EQ occurs', fontsize=8,rotation=90, rotation_mode='anchor')    

    plt.xlabel('Time, days', fontsize=10, **titles_font)
    plt.ylabel('LOS, hours', fontsize=10, **titles_font)
    # plt.title('Lenght of Stay - Scenario '+str(Scenario_number), fontsize=10, **titles_font)
    leg = plt.legend(fontsize=8, loc="upper right")
    leg.get_frame().set_edgecolor('0.0')
    leg.get_frame().set_linewidth(0.75)
    # plt.grid()
    plt.xticks([0,2,4,6,8,10,12,14], fontsize=8)
    # plt.xticks([0,5,10,15,20,25,30], fontsize=8)
    plt.yticks([0,5,10,15,20,25], fontsize=8)
    plt.xlim([0, 14])
    plt.ylim([0, 28])
    path = results_plots_path+'\All\Sim'+str(Scenario_number)+'_LOS'
    plt.savefig(path, bbox_inches="tight", dpi =300)
    plt.show()

def plot_LOS_all(time, Total_LOS, Sim_number, Scenario_number):
    
    titles_font = {'fontname':'Arial'}
    time2 = [t/24 for t in time]
    
    # Get the mean
    columns = len (Total_LOS[0])
    mean_Total_LOS = np.zeros(columns)
    for i in range(columns):
        mean_Total_LOS[i] = np.mean(Total_LOS[:,i])
    
    limit = []
    for i in range(Sim_number):
        plt.plot(time2, Total_LOS[i,:], color = '0.3', linewidth=1.0, alpha=0.25)
        limit.append(max(Total_LOS[i,:]))
            
    plt.plot([0,50], [0,0], color = '0.3', linewidth=1.0, alpha=0.25,
             label = 'Simulation '+str(Scenario_number)+'-'+str(Scenario_number+Sim_number-1))
    # plt.plot([0,30], [10,10], color = 'black', linewidth=1.5, linestyle='--', label = 'LOS limit')
    plt.plot(time2, mean_Total_LOS, color = 'black', linewidth=1.0, label = 'Mean LOS')
    plt.plot([0,200], [10,10], color = 'red', linewidth=1.5, linestyle='--', label = 'Limit')
    
    # EQ occurs at day 2
    plt.plot([2,2], [0,500], color = 'black', linewidth=0.75, linestyle='--') 
    l1 = np.array((3.4, 19.1))
    plt.text(*l1, 'EQ occurs', fontsize=8,rotation=90, rotation_mode='anchor')    

    plt.xlabel('Time, days', fontsize=10, **titles_font)
    plt.ylabel('LOS, hours', fontsize=10, **titles_font)
    # plt.title('ED Length of Stay - Scenarios 1 to 30', fontsize=10, **titles_font)
    leg = plt.legend(fontsize=8, loc="upper right")
    leg.get_frame().set_edgecolor('0.0')
    leg.get_frame().set_linewidth(0.75)
    # plt.grid()
    
    plt.xticks([0,5,10,15,20,25,30], fontsize=8)
    plt.yticks([0,5,10,15,20,25,30], fontsize=8)
    plt.xlim([0, 30])
    plt.ylim([0, 30]) #27 
    path = results_plots_path+'\Patient_LOS'
    plt.savefig(path, bbox_inches="tight", dpi =300)
    plt.show()
    
    
def plot_waiting_times(WT_plot, time, Sim_number, Scenario_number, title):
    
    titles_font = {'fontname':'Arial'}
    
    if Sim_number == 1:
        
        plt.plot(time, WT_plot, color = 'tab:blue', linewidth=1.5, label = 'Simulation '+str(Scenario_number))
        plt.plot([0,time[-1]], [2,2], color = 'black', linewidth=2.0, linestyle= '--', label="Limit")
        
        # EQ occurs at day 2
        plt.plot([2,2], [0,500], color = 'black', linewidth=0.75, linestyle='--') 
        l1 = np.array((3.4, 2.55))
        plt.text(*l1, 'EQ occurs', fontsize=8,rotation=90, rotation_mode='anchor')    
        
        plt.xlabel('Time, days', fontsize=10, **titles_font)
        plt.ylabel(str(title)+' Wait Time, hours', fontsize=10, **titles_font)
        # plt.title(str(title)+' Wait Times', fontsize=10, **titles_font)
        leg = plt.legend(fontsize=8, loc="upper right")
        leg.get_frame().set_edgecolor('0.0')
        leg.get_frame().set_linewidth(0.75)
        # plt.grid()
        
        plt.xticks([0,5,10,15,20,25,30], fontsize=8)
        plt.yticks([0,2,4,6,8,10,12], fontsize=8)
        plt.xlim([0, time[-1]])
        plt.ylim([0, max(4,math.ceil(max(WT_plot)))])
        path = results_plots_path+'\All\Sim'+str(Scenario_number)+'_WT_'+str(title.replace(" ", "_"))
        plt.savefig(path, bbox_inches="tight", dpi =300)
        plt.show()

    else:
        
        limit = []
        # Get the mean
        columns = len (WT_plot[0])
        mean_WT = np.zeros(columns)
        for i in range(columns):
            mean_WT[i] = np.mean(WT_plot[:,i])
               
        
        for i in range(Sim_number):
            plt.plot(time, WT_plot[i,:], color = '0.3', linewidth=1.0, alpha=0.25)
            limit.append(max(WT_plot[i,:]))
        plt.plot([0,50], [0,0], color = '0.3', linewidth=1.0, alpha=0.25,
                 label = 'Simulation '+str(Scenario_number)+'-'+str(Scenario_number+Sim_number-1))
        plt.plot(time, mean_WT, color = 'black', linewidth=1.0, label = 'Mean WT')
        plt.plot([0,time[-1]], [2,2], color = 'red', linewidth=1.5, linestyle= '--', label="Limit")
        
        # EQ occurs at day 2
        plt.plot([2,2], [0,500], color = 'black', linewidth=0.75, linestyle='--') 
        if max(4,math.ceil(max(limit))) == 4:
            loc = 2.52
        else:
            loc = 5.75 # 5.1 # 7.6 # 11.2
        l1 = np.array((3.4, loc))
        plt.text(*l1, 'EQ occurs', fontsize=8,rotation=90, ha='left', rotation_mode='anchor')
        
        plt.xlabel('Time, days', fontsize=10, **titles_font)
        plt.ylabel(str(title)+' WT, hours', fontsize=10, **titles_font)
        # plt.title(str(title)+' Wait Times', fontsize=10, **titles_font)
        leg = plt.legend(fontsize=8, loc="upper right")
        leg.get_frame().set_edgecolor('0.0')
        leg.get_frame().set_linewidth(0.75)
        # plt.grid()
        plt.xticks([0,5,10,15,20,25,30], fontsize=8)
        plt.yticks([0,2,4,6,8,10,12,14,16,18,20,22], fontsize=8)
        plt.xlim([0, 30])
        plt.ylim([0, max(4,math.ceil(max(limit)))])
        path = results_plots_path+'\WT_'+str(title.replace(" ", "_"))
        plt.savefig(path, bbox_inches="tight", dpi =300)
        plt.show()

def plot_all_waiting_times(Triage_WT, ER_WT, OR_WT, Imaging_WT, Diagnostic_WT, Beds_WT, time, Scenario_number):
    
    # Plot all WT for one single simulation
    
    titles_font = {'fontname':'Arial'}
    
    plt.plot(time, Triage_WT, color = 'tab:green', linewidth=1.5, label = 'Triage')
    plt.plot(time, ER_WT, color = 'tab:blue', linewidth=1.5, label = 'ERs')
    plt.plot(time, OR_WT, color = 'tab:orange', linewidth=1.5, label = 'ORs')
    plt.plot(time, Imaging_WT, color = 'tab:brown', linewidth=1.5, label = 'Labs')
    # plt.plot(time, Diagnostic_WT, linewidth=1.5, label = 'Diagnostic WT')
    plt.plot(time, Beds_WT, color = 'tab:red', linewidth=1.5, label = 'Beds')
    plt.plot([0,time[-1]], [2,2], color = 'black', linewidth=2.0, linestyle= '--', label="Limit")
    
    # EQ occurs at day 2
    plt.plot([2,2], [0,500], color = 'black', linewidth=0.75, linestyle='--') 
    l1 = np.array((2.7, 5.0)) #  3.4  / 5.0, 2.55
    plt.text(*l1, 'EQ occurs', fontsize=8,rotation=90, rotation_mode='anchor')    
    
    plt.xlabel('Time, days', fontsize=10, **titles_font)
    plt.ylabel('WT, hours', fontsize=10, **titles_font)
    # plt.title('Wait Times - Scenario '+str(Scenario_number), fontsize=10, **titles_font)
    leg = plt.legend(fontsize=8, loc="upper right")
    leg.get_frame().set_edgecolor('0.0')
    leg.get_frame().set_linewidth(0.75)
    # plt.grid()
    
    plt.xticks([0,2,4,6,8,10,12,14], fontsize=8)
    plt.yticks([0,2,4,6,8,10,12], fontsize=8)
    plt.xlim([0, 14])
    limit = max( max(Triage_WT), max(ER_WT), max(OR_WT), max(Imaging_WT), max(Diagnostic_WT), max(Beds_WT) )
    plt.ylim([ 0, max(8,math.ceil(limit)) ])
    path = results_plots_path+'\All\Sim'+str(Scenario_number)+'_all_WT'
    plt.savefig(path, bbox_inches="tight", dpi =300)
    plt.show()


def plot_DvsC(Demand_SS_plot,Demand_EQ_plot,Demand_Total_plot, time, Sim_number, Scenario_number, capacity_red):
    
    titles_font = {'fontname':'Arial'}   
    
    if Sim_number == 1:    
        
        plt.plot(time, Demand_SS_plot, color = 'tab:green', linewidth=0.5, label = 'Steady-State Demand')
        plt.plot(time, Demand_EQ_plot, color = 'tab:red', linewidth=0.5, label = 'Earthquake Demand')
        plt.plot(time, Demand_Total_plot, color = 'tab:blue', linewidth=1.0, label = 'ED Demand')
        plt.plot([0,2,2.01,time[-1]], [14.3,14.3,capacity_red,capacity_red], color = 'black', linewidth=2.0, label="ED Capacity")
        
        # EQ occurs at day 2
        plt.plot([2,2], [0,500], color = 'black', linewidth=0.75, linestyle='--') 
        l1 = np.array((1.88, 25))
        plt.text(*l1, 'EQ occurs', fontsize=8,rotation=90, rotation_mode='anchor')    
        
        plt.xlabel('Time, days', fontsize=10, **titles_font)
        plt.ylabel('Arrival Rate, patients/hour', fontsize=10, **titles_font)
        # plt.title('ED Demand & Capacity - Scenario '+ str(Scenario_number), fontsize=10, **titles_font)
        leg = plt.legend(fontsize=8, loc="upper right")
        leg.get_frame().set_edgecolor('0.0')
        leg.get_frame().set_linewidth(0.75)
        # plt.grid()
        plt.xticks([0,1,2,3,4,5,6,7], fontsize=8)
        # plt.xticks([0,2,4,6,8,10,12,14], fontsize=8)
        plt.yticks([0,10,20,30,40], fontsize=8)
        plt.xlim([0, 7])
        plt.ylim([0, 40])
        path = results_plots_path+'\All\Sim'+str(Scenario_number)+'_D&C'
        plt.savefig(path, bbox_inches="tight", dpi =300)
        plt.show()    
             
    else:        
        
        for i in range(Sim_number):
            plt.plot(time, Demand_Total_plot[i,:], color = '0.25', linewidth=1.0, alpha=0.25)
            capacity_red = functionality_data[Sim_first+i-1,5]
            plt.plot([0,2,2.01,time[-1]], [14.3,14.3,capacity_red,capacity_red], color = 'red', linewidth=1.0, alpha=0.25)

        # plt.plot([0,15], [200,200], color = '0.25', linewidth=1.0, alpha=0.25, label = 'Simulations '+str(Scenario_number)+'-'+str(Scenario_number+Sim_number-1))
        plt.plot([0,15], [0,0], color = '0.25', linewidth=1.0, alpha=0.25, label = 'ED Demand')
        plt.plot([0,15], [0,0], color = 'red', linewidth=1.0, alpha=0.25, label = 'ED Capactity')
        
        # EQ occurs at day 2
        plt.plot([2,2], [0,500], color = 'black', linewidth=0.75, linestyle='--') 
        l1 = np.array((2.7, 2.22))
        plt.text(*l1, 'EQ occurs', fontsize=8,rotation=90, rotation_mode='anchor')    

        plt.xlabel('Time, days', fontsize=10, **titles_font)
        plt.ylabel('Arrival Rate, patients/hour', fontsize=10, **titles_font)
        # plt.title('ED Demand & Capacity - Scenarios 1 to 30', fontsize=10, **titles_font)
        leg = plt.legend(fontsize=8, loc="upper right")
        leg.get_frame().set_edgecolor('0.0')
        leg.get_frame().set_linewidth(0.75)
        # plt.grid()
        plt.xticks([0,2,4,6,8,10,12,14], fontsize=8)
        plt.yticks([0,10,20,30,40], fontsize=8)
        plt.xlim([0, 15])
        plt.ylim([0, 45])
        path = results_plots_path+'\Demand_vs_Capacity'
        plt.savefig(path, bbox_inches="tight", dpi =300)
        plt.show()  
        
def plot_DtoC(D_to_C, time, Sim_number, Scenario_number, functionality_data):
    
    titles_font = {'fontname':'Arial'}   
    
    if Sim_number == 1:  

        plt.plot(time, D_to_C, color = '0.0', linewidth=1.0, label = 'DCR')
        plt.plot([0,200], [1,1], color = 'red', linewidth=1.5, linestyle='--', label = 'Limit')
        
        # EQ occurs at day 2
        plt.plot([2,2], [0,500], color = 'black', linewidth=0.75, linestyle='--') 
        l1 = np.array((1.85, 1.9)) # 2.7 , 1.9
        plt.text(*l1, 'EQ occurs', fontsize=8,rotation=90, rotation_mode='anchor')    
        
        plt.xlabel('Time, days', fontsize=10, **titles_font)
        plt.ylabel('DCR', fontsize=10, **titles_font)
        # plt.title('ED Demand to Capacity Ratio - Scenario '+ str(Scenario_number), fontsize=10, **titles_font)
        leg = plt.legend(fontsize=8, loc="upper right")
        leg.get_frame().set_edgecolor('0.0')
        leg.get_frame().set_linewidth(0.75)
        # plt.grid()
        plt.xticks([0,1,2,3,4,5,6,7], fontsize=8)
        # plt.xticks([0,2,4,6,8,10,12,14], fontsize=8)
        plt.yticks([0,1,2,3,4,5], fontsize=8)
        plt.xlim([0, 7]) # for scenario 17, [0, 7], otherwise, [0, 15]
        plt.ylim([0, max(3,math.ceil(max(D_to_C)))])  
        path = results_plots_path+'\All\Sim'+str(Scenario_number)+'_DtoC'
        plt.savefig(path, bbox_inches="tight", dpi =300)
        plt.show()

    
    else:  
             
        # Get Mean D/C
        columns = len (D_to_C[0])
        mean_D_to_C = np.zeros(columns)
        for i in range(columns):
            mean_D_to_C[i] = np.mean(D_to_C[:,i])
        
        # # Get Median in Demand
        # max_D_to_C = []
        # for i in range(Sim_number):
        #     max_D_to_C.append(max(D_to_C[i]))       
        # median = statistics.median_high(max_D_to_C)
        # median_simulation = max_D_to_C.index(median)
        
        # # Get Median in Capacity
        # max_D_to_C = []
        # for i in range(Sim_number):
        #     max_D_to_C.append(max(D_to_C[i,96:]))       
        # median = statistics.median_high(max_D_to_C)
        # median_simulation = max_D_to_C.index(median)
        
        # Plots
        for i in range(Sim_number): 
            plt.plot(time, D_to_C[i,:], color = '0.25', linewidth=0.5, alpha=0.25)
        
        plt.plot([0,15], [0,0], color = '0.25', linewidth=0.5, alpha=0.25, label = 'Indiv. Runs - DCR')
        # plt.plot(time, D_to_C[median_simulation,:], color = 'black', linewidth=0.75, label = 'Median D/C ratio')
        plt.plot(time, mean_D_to_C, color = 'black', linewidth=1.0, label = 'Mean DCR')
        # plt.plot([0,15], [200,200], color = 'red', linewidth=1.0, alpha=0.25, label = 'ED Capactity')
        plt.plot([0,200], [1,1], color = 'red', linewidth=1.5, linestyle='--', label = 'Limit')
        
        # EQ occurs at day 2
        plt.plot([2,2], [0,500], color = 'black', linewidth=0.75, linestyle='--') 
        l1 = np.array((1.85, 2.2)) # 2.5
        plt.text(*l1, 'EQ occurs', fontsize=8,rotation=90, rotation_mode='anchor')    
        
        plt.xlabel('Time, days', fontsize=10, **titles_font)
        plt.ylabel('DCR', fontsize=10, **titles_font)
        # plt.title('ED Demand to Capacity Ratio - Scenarios 1 to 30', fontsize=10, **titles_font)
        leg = plt.legend(fontsize=8, loc="upper right")
        leg.get_frame().set_edgecolor('0.0')
        leg.get_frame().set_linewidth(0.75)
        # plt.grid()
        plt.xticks([0,2,4,6,8,10,12,14], fontsize=8)
        plt.yticks([0,1,2,3,4,5], fontsize=8)
        plt.xlim([0, 15])
        plt.ylim([0, max(3.5,math.ceil(max(D_to_C_plot[0,:])))])  
        # plt.ylim([0, 3.5]) 
        path = results_plots_path+'\Demand_to_Capacity_Ratio'
        plt.savefig(path, bbox_inches="tight", dpi =300)
        plt.show()

        
def update_VGH_results1(Jaamsim_output1):
    GM_MC_results = "VGH_results_summary.dat"
    with open(Jaamsim_output1, "r") as f:
        new_outputs = f.readlines()[1]
    with open(GM_MC_results, "a") as f:
        f.write(new_outputs)
    return

def update_VGH_results2(Jaamsim_output2,SimNumber):
    GM_MC_results = "VGH_results_WT.dat"
    with open(Jaamsim_output2, "r") as f:
        new_outputs = f.readlines()[11:]
    with open(GM_MC_results, "a") as f:
        line = 'Simulation ' +str(SimNumber)+'\n'
        f.write(line)
        f.write('Time[min]  OR_wt[h]  ER_wt[h]  Beds_wt[h]  Triage_wt[h]  Imag_wt[h]  Diag_wt[h]  Demand[pat./h]\n')
        f.write(''.join(new_outputs))
        f.write('\n')
    return

def clean_VGH_results_file1(file):
    with open(file, "r") as f:
        keep_line = f.readlines()[0]
    with open(file, "w") as f:
        f.write(keep_line)
    return

def clean_VGH_results_file2(file):
    file = open(file,"w")
    file.close()
    return

# =============================================================================
# Run Simulation
# =============================================================================

# MC_simulations = np.int64(functionality_data[-1,0])
Sim_first = 1 # if do not wanna run everything 
Sim_last = 30
Sim_number = int(Sim_last-Sim_first+1)
# Select file (scenario: night, transit, day)
Casualty_file = 'Scenario_Casualty_Estimation_Day.txt' # Scenario_Casualty_Estimation_Day_NoS1.txt

# Base model
Jaamsim_model = "VGH_Emergency_Department_capacity_python.cfg"
# Jaamsim_base_model = "VGH_Emergency_Department_capacity2.cfg"

# parameters
duration = 720 # [h] / 360 h = 30 d

# Read file
# functionality_data = np.loadtxt('FunctionalityFactors.txt',delimiter = '\t', skiprows=1)
# file = r'C:\Users\geral\Documents\Research\Pelicun\VGH_ED\Functionality_results\Functionality_Results.csv'
# file = r'C:\Users\geral\Documents\Research\Pelicun\VGH_ED\Functionality_results\Functionality_Results_Yu_et_al.csv'
file = functionality_file
a = pd.read_csv(file,sep=",",skiprows=0)
functionality_data = a.values

# Clean output files
clean_VGH_results_file1('VGH_results_summary.dat') 
clean_VGH_results_file2('VGH_results_WT.dat') 

interval = 12 # calculate avg every 24h
total_LOS_plot = np.zeros(shape=(Sim_number,int(duration/interval)))

for Scenario in range(Sim_first, Sim_last+1):
    
    j = Scenario-1
    Factor1 = functionality_data[j,1]
    Factor2 = functionality_data[j,2]
    Factor3 = functionality_data[j,3]
    Factor4 = functionality_data[j,4]
    
    parameters1=[('Simulation RunIndexDefinitionList', Scenario),\
                ('Simulation StartingRunNumber', Scenario ),\
                ('Simulation EndingRunNumber', Scenario ),\
                ('Beds Capacity', 450 ),\
                ('RunInputs DataFile', Casualty_file)]

    # parameters2=[('NormalDistribution1 MinValue', 0 ),\
    #             ('NormalDistribution1 MaxValue', temp_mean[i] *2 ),\
    #             ('NormalDistribution1 Mean', temp_mean[i] ),\
    #             ('NormalDistribution1 StandardDeviation', 2.0 )]
    
    parameters3=[('Triage_Resources Value', 4, 4 ),\
                ('ER_Resources Value', 40, 40*Factor1 ),\
                ('OR_Resources Value', 23, 23*Factor3 ),\
                ('RR_Resources Value', 23, 23*Factor3 )]
        
    # Write Jaamsim script to run
    get_modified_script(Jaamsim_model, parameters1, parameters3) # you can add parameters2

    # Select Model to Run
    run_simulation(Jaamsim_model) # Jaamsim_base_model or Jaamsim_model
    
    # Save Outputs
    Jaamsim_output1 = "VGH_Emergency_Department_capacity_python.dat" # Demand summary, discharge & Max WT. 
    update_VGH_results1(Jaamsim_output1) # save it in 'VGH_results_summary'
    Jaamsim_output2 = "VGH_Emergency_Department_capacity_python-WaitingTimes.log" # Wait Times for OR, ER, beds. 
    update_VGH_results2(Jaamsim_output2,Scenario) # save it in 'VGH_results_summary'
    
    # Save & Plot LOS
    LOS_S1_file = "VGH_Emergency_Department_capacity_python-Discharge_S1.log"
    LOS_S2_file = "VGH_Emergency_Department_capacity_python-Discharge_S2.log"
    LOS_S3_file = "VGH_Emergency_Department_capacity_python-Discharge_S3.log"
    time_S1, LOS_S1, num_S1_patients = get_avg_LOS(LOS_S1_file)
    time_S2, LOS_S2, num_S2_patients = get_avg_LOS(LOS_S2_file)
    time_S3, LOS_S3, num_S3_patients = get_avg_LOS(LOS_S3_file)
    Total_patients = num_S1_patients+ num_S2_patients+ num_S3_patients
    Total_LOS = []
    for i in range(len(time_S1)):
        avg = (LOS_S1[i]*num_S1_patients + LOS_S2[i]*num_S2_patients + LOS_S3[i]*num_S3_patients) / Total_patients
        Total_LOS.append(avg)
    
    k = Scenario-Sim_first
    total_LOS_plot[k,:] = Total_LOS # save LOS outputs to plot later
    plot_LOS(time_S1, LOS_S1, LOS_S2, LOS_S3, Total_LOS, Scenario) # plot LOS 
    
    # arrival_capacity = min_arrival_rate + interval*(i-2)
    print ('-----------------------------------------')
    print ('Scenario '+str(Scenario)+' Done')
        
# =============================================================================
# Plots
# =============================================================================

plt.rcParams["figure.figsize"] = (3.25,2.20)
plt.rcParams['axes.linewidth'] = 1.25 

Sim_number = int(Sim_last-Sim_first+1)
WT_file = 'VGH_results_WT.dat'
N_points = int(duration/1)+1 # interval 1h

OR_WT_plot = np.zeros(shape=(Sim_number,N_points)) 
ER_WT_plot = np.zeros(shape=(Sim_number,N_points)) 
Beds_WT_plot = np.zeros(shape=(Sim_number,N_points))
Triage_WT_plot = np.zeros(shape=(Sim_number,N_points)) 
Imaging_WT_plot = np.zeros(shape=(Sim_number,N_points)) 
Diagnostic_WT_plot = np.zeros(shape=(Sim_number,N_points)) 
Demand_SS_plot = np.zeros(shape=(Sim_number,N_points))
Demand_EQ_plot = np.zeros(shape=(Sim_number,N_points)) 
Demand_Total_plot = np.zeros(shape=(Sim_number,N_points)) 
D_to_C_plot = np.zeros(shape=(Sim_number,N_points)) 

for k in range(Sim_number):
    
    time,OR_WT,ER_WT,Beds_WT,Triage_WT,Imaging_WT,Diagnostic_WT,Demand_SS,Demand_EQ,Demand_Total = get_waiting_times(WT_file, N_points, k)  # k
   
    OR_WT_plot[k,:] = OR_WT
    ER_WT_plot[k,:] = ER_WT
    Beds_WT_plot[k,:] = Beds_WT
    Triage_WT_plot[k,:] = Triage_WT
    Imaging_WT_plot[k,:] = Imaging_WT
    Diagnostic_WT_plot[k,:] = Diagnostic_WT
    Demand_SS_plot[k,:] = Demand_SS
    Demand_EQ_plot[k,:] = Demand_EQ
    Demand_Total_plot[k,:] = Demand_Total
    capacity_red = functionality_data[Sim_first+k-1,5]
    # Demand to Capacity
    Demand1 = Demand_Total[:48]
    Demand2 = Demand_Total[48:]       
    Capacity1 = [r/14.3 for r in Demand1]
    Capacity2 = [r/capacity_red for r in Demand2]
    D_to_C = Capacity1 + Capacity2
    D_to_C_plot[k,:] = D_to_C
    
    
# =============================================================================
# Individual Plots
# =============================================================================
    
    plot_DvsC(Demand_SS,Demand_EQ,Demand_Total, time, 1, Sim_first+k, capacity_red)  
    plot_DtoC(D_to_C, time, 1, Sim_first+k, capacity_red)
    plot_all_waiting_times(Triage_WT,ER_WT,OR_WT,Imaging_WT,Diagnostic_WT,Beds_WT,time, Sim_first+k)
    # plot_waiting_times(OR_WT, time, 1, Sim_first+k, 'Operating Room')
    # plot_waiting_times(ER_WT, time, 1, Sim_first+k, 'Emergency Room')
    # plot_waiting_times(Beds_WT, time, 1, Sim_first+k, 'Bed')
    # plot_waiting_times(Triage_WT, time, 1, Sim_first+k, 'Triage')
    # plot_waiting_times(Imaging_WT, time, 1, Sim_first+k, 'Imaging Lab')
    # plot_waiting_times(Diagnostic_WT, time, 1, Sim_first+k, 'Diagnostic Lab')

       
# =============================================================================
# Plot all
# =============================================================================

if Sim_number > 1:
    
    plot_LOS_all(time_S1, total_LOS_plot, Sim_number, Sim_first)
    plot_waiting_times(OR_WT_plot, time, Sim_number,Sim_first, 'OR')
    plot_waiting_times(ER_WT_plot, time, Sim_number,Sim_first, 'ER')
    # plot_waiting_times(Beds_WT_plot, time, Sim_number,Sim_first, 'Bed')
    # plot_waiting_times(Triage_WT_plot, time, Sim_number,Sim_first, 'Triage')
    # plot_waiting_times(Imaging_WT_plot, time, Sim_number,Sim_first, 'Imaging Lab')
    # plot_waiting_times(Diagnostic_WT_plot, time, Sim_number,Sim_first, 'Diagnostic Lab')
    # plot_DvsC(Demand_SS_plot,Demand_EQ_plot,Demand_Total_plot, time, Sim_number,Sim_first, functionality_data)
    plot_DtoC(D_to_C_plot, time, Sim_number, Sim_first, functionality_data)











